#!/usr/bin/env python

from __future__ import unicode_literals

import os
import sys

os.environ['DJANGO_SETTINGS_MODULE'] = 'mailer.settings.test'
sys.path.append('./src')

from django.conf import settings
from django.test.utils import get_runner

if __name__ == "__main__":
    from django import setup
    setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(["tests"])
    sys.exit(bool(failures))



from django.contrib import admin                                                                                                                  
from django.utils.translation import ugettext_lazy

from .models import Address, Signature, Mail, MailBody

def clone_selected(modeladmin, request, queryset):
    for model in queryset.all():
        model.clone()
clone_selected.short_description =  ugettext_lazy("Clone selected %(verbose_name_plural)s")

class AddressAdmin(admin.ModelAdmin):
    list_display = ('address', 'comment')

class SignatureAdmin(admin.ModelAdmin):
    list_display = ('name',)
    actions = [clone_selected]
    prepopulated_fields = {'name': ('sig',)}

class MailAdmin(admin.ModelAdmin):
    list_display = ('name', 'subject', 'sender')
    actions = [clone_selected]

class MailBodyAdmin(admin.ModelAdmin):
    list_display = ('name', 'subject')
    actions = [clone_selected]
    prepopulated_fields = {'name': ('subject',)}

admin.site.register(Address, AddressAdmin)
admin.site.register(Signature, SignatureAdmin)
admin.site.register(Mail, MailAdmin)
admin.site.register(MailBody, MailBodyAdmin)



from __future__ import unicode_literals

from django.test import TestCase

from mailrobot.models import Mail, MailBody, Signature, Address
from mailrobot.models import MailrobotError, MailrobotNoSenderError, MailrobotNoRecipientsError


HELLO_WORLD_BODY_TEMPLATE = """Hello, {{ world|default:"World" }}!

This is a test of mail robot."""

HELLO_WORLD_SUBJECT_TEMPLATE = 'Hello, {{ world|default:"World" }}!'

class CloneTest(TestCase):

    def setUp(self):
        self.mailbody = MailBody(subject='test', body='test', name='test')
        self.mailbody.save()
        self.mail = Mail(content=self.mailbody, name='test')
        self.mail.save()
        self.address = Address(address='postmaster@example.com')
        self.address.save()
        self.name_max_length = MailBody.NAME_MAX_LENGTH

    def test_default_clone(self):
        model = self.mailbody
        clone = model.clone()
        self.assertNotEqual(model, clone)

    def test_cloned_long_name(self):
        longname = 'x' * MailBody.NAME_MAX_LENGTH
        model = self.mailbody
        model.name = longname
        clone = model.clone()
        self.assertNotEqual(model, clone)
        self.assertLessEqual(len(clone.name), MailBody.NAME_MAX_LENGTH)

    def test_mail_clone(self):
        model = self.mail
        model.sender = self.address
        model.recipients.set([self.address])
        model.save()
        clone = model.clone()
        self.assertNotEqual(model, clone)
        self.assertEqual(set(clone.recipients.all()), set(model.recipients.all()))

class MailTest(TestCase):

    def setUp(self):
        self.mailbody = MailBody(
            subject=HELLO_WORLD_SUBJECT_TEMPLATE,
            body=HELLO_WORLD_BODY_TEMPLATE,
            name='hello-world',
        )
        self.mailbody.save()

        self.empty_sig = Signature(name='empty', sig='').save()
        self.simple_sig = Signature(name='simple', sig='simple').save()
        self.complex_sig = Signature(name='complex', sig='{{ complex }}').save()

        self.mail = Mail(
            content=self.mailbody,
            name='hello-world',
        )
        self.mail.save()

        self.simple_address = Address(
            address='postmaster@example.com'
        )
        self.simple_address.save()

        self.complex_address = Address(
            address='abuse@example.com',
            comment='CERT'
        )
        self.complex_address.save()

    def test_no_signature(self):
        expected_result = ''
        self.assertEqual('', self.mail.attach_signature())

    def test_no_sender_no_recipients(self):
        with self.assertRaises(MailrobotError):
            self.mail.make_message()

    def test_has_sender_lacks_recipients(self):
        with self.assertRaises(MailrobotNoRecipientsError):
            self.mail.make_message(sender=self.simple_address)

    def test_no_sender_one_recipient(self):
        with self.assertRaises(MailrobotNoSenderError):
            self.mail.make_message(recipients=[self.simple_address])

    def test_validate_one_simple_recipient(self):
        self.mail.sender = self.simple_address
        self.mail.save()
        recipients = (self.simple_address,)
        result = self.mail.validate_addresses(recipients=recipients)
        self.assertTrue(result)

    def test_validate_one_complex_recipient(self):
        self.mail.sender = self.simple_address
        self.mail.save()
        recipients = (self.complex_address,)
        result = self.mail.validate_addresses(recipients=recipients)
        self.assertTrue(result)

    def test_validate_two_recipients(self):
        self.mail.sender = self.simple_address
        self.mail.save()
        recipients = (self.simple_address, self.complex_address)
        result = self.mail.validate_addresses(recipients=recipients)
        self.assertTrue(result)

class SignatureTest(TestCase):

    SIG_TEMPLATE = '\r\n\r\n\r\n-- \r\n%s'

    def test_empty_signature(self):
        sig = Signature(name='test1', sig='')
        sig.save()
        expected_result = ''
        self.assertEqual(expected_result, sig.attach())

    def test_simple_signature(self):
        content = 'foo'
        sig = Signature(name='test1', sig=content)
        expected_result = self.SIG_TEMPLATE % content
        self.assertEqual(expected_result, sig.attach())

    def test_complex_signature(self):
        content = 'foo{{ word }}foo'
        context = {'word': 'foo'}
        sig = Signature(name='test1', sig=content)
        expected_result = self.SIG_TEMPLATE % 'foofoofoo'
        self.assertEqual(expected_result, sig.attach(context))



from .base import *

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'mailer.sqlite3',                      # Or path to database file if using sqlite3.
    }
}

SECRET_KEY = 'na2Tei0FoChe3ooloh5Yaec0ji7Aipho'

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'



import sys, os, os.path

_HOME = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(_HOME)
os.environ["DJANGO_SETTINGS_MODULE"] = "mailer.settings.demo"

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()



from __future__ import unicode_literals

import time
from copy import deepcopy
import warnings

from django.db import models
from django.template import Context, Template
from django.core.mail import EmailMessage


def _render_from_string(templatestring, context=None):
    if context is None:
        context = {}
    template = Template(templatestring)
    return template.render(Context(context))


class MailrobotError(ValueError):
    pass


class MailrobotNoSenderError(MailrobotError):
    pass


class MailrobotNoRecipientsError(MailrobotError):
    pass


class NameManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class AbstractNamedModel(models.Model):
    NAME_MAX_LENGTH = 40
    name = models.SlugField(max_length=NAME_MAX_LENGTH, unique=True)

    objects = NameManager()

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    def natural_key(self):
        return self.name

    def get_suffixed_name(self, name, suffix):
        "Return a name shorter than NAME_MAX_LENGTH, with -<suffix> at the end"
        suffix = ''.join((list(reversed('%s' % suffix))))
        suffixedname = '%s-%s' % (name[:-len(suffix)-1], suffix)
        return suffixedname[:self.NAME_MAX_LENGTH]

    def clone(self):
        "Clone and return a Named model"

        newself = deepcopy(self)
        newself.pk = None
        newself.name = self.get_suffixed_name(self.name, int(time.time()))
        assert newself.name != self.name, "Names are identical"
        newself.save(force_insert=True)
        return newself


class AddressManager(models.Manager):
    def get_by_natural_key(self, address):
        return self.get(address=address)


class Address(models.Model):
    address = models.EmailField(unique=True)
    comment = models.CharField(max_length=66, blank=True, null=True)

    objects = AddressManager()

    class Meta:
        verbose_name_plural = 'addresses'

    def __str__(self):
        if self.comment:
            return '%s <%s>' % (self.comment, self.address)
        return self.address

    def natural_key(self):
        return self.address


class Signature(AbstractNamedModel):
    """Email signature

    No validation as to size.
    """

    sig = models.TextField()

    def attach(self, context=None):
        """
        The signature is attached like so:

        ::

                the final line of content


                --
                signature
        """

        signature = _render_from_string(self.sig, context)
        if signature:
            return '\r\n\r\n\r\n-- \r\n%s' % signature
        return ''


class MailBody(AbstractNamedModel):
    "Subject and bodytext of the email"

    subject = models.CharField(max_length=66)
    body = models.TextField()

    class Meta:
        verbose_name_plural = 'mailbodies'


class Mail(AbstractNamedModel):
    """Canned Mail with default sender, Reply-To and recipients

    Verifies that there is a sender and at least one recipient.
    """

    KEYFIELD_DEFAULTS = {
        'null': True,
        'blank': True,
    }

    M2M_KEYFIELD_DEFAULTS = {
        'blank': True,
    }

    content = models.ForeignKey(MailBody, on_delete=models.CASCADE,
                                related_name='mail')
    signature = models.ForeignKey(Signature, on_delete=models.CASCADE,
                                  related_name='mail', **KEYFIELD_DEFAULTS)
    sender = models.ForeignKey(Address, on_delete=models.CASCADE,
                               related_name='sender', **KEYFIELD_DEFAULTS)
    recipients = models.ManyToManyField(Address, related_name='recipients',
                                        **M2M_KEYFIELD_DEFAULTS)
    ccs = models.ManyToManyField(Address, related_name='cc',
                                 **M2M_KEYFIELD_DEFAULTS)
    bccs = models.ManyToManyField(Address, related_name='bcc',
                                  **M2M_KEYFIELD_DEFAULTS)
    reply_to = models.ForeignKey(Address, on_delete=models.CASCADE,
                                 related_name='reply_to', **KEYFIELD_DEFAULTS)

    def clone(self):
        """Clone and return a Mail

        Use this to send the same MailBody to several sets of recipients.
        """

        newself = super(Mail, self).clone()
        newself.recipients.set(self.recipients.all())
        newself.ccs.set(self.ccs.all())
        newself.bccs.set(self.bccs.all())
        return newself

    @property
    def subject(self):
        return self.content.subject

    @property
    def body(self):
        return self.content.body

    def attach_signature(self, context=None):
        "Attach signature, if any"

        if self.signature:
            return self.signature.attach(context)
        return ''

    def make_content(self, context=None):
        "Generate the content (body + signature) from django templates and context"

        body = _render_from_string(self.content.body, context)
        return body + self.attach_signature()

    def make_subject(self, context=None):
        "Generate the subject from a django template and context"

        return _render_from_string(self.content.subject, context)

    def get_reply_to(self, reply_to=''):
        "Reply-To may be empty"

        if not reply_to and self.reply_to:
            return str(self.reply_to)
        return reply_to

    def get_sender(self, sender=None):
        "Sender may not be empty"

        if not sender:
            if self.sender:
                return str(self.sender)
        else:
            return sender
        raise MailrobotNoSenderError("Mail must have a sender")

    def _get_addresses(self, attribute, additional=(), required=False):
        "Merge the addresses of field <attribute> with list in <additional>"

        attribute = getattr(self, attribute)
        addresses = ()
        if attribute:
            addresses = set([str(row) for row in attribute.all()])
            addresses = addresses | set(additional)
        if required and not addresses:
            raise MailrobotNoRecipientsError('No recipient addresses!')
        return addresses

    def get_recipients(self, additional=(), required=True):
        """Get recipients for To: and ensures there is at least one.

        Email lacking anything in To: is likely spam."""

        try:
            return self._get_addresses('recipients', additional, required)
        except MailrobotNoRecipientsError as e:
            warnings.warn('Beware: %s An empty "To:" looks spammy' % e.args[0])
            return self._get_addresses('recipients', additional, required=False)

    def get_ccs(self, additional=()):
        """Get recipients for CC:

        May be empty."""

        return self._get_addresses('ccs', additional)

    def get_bccs(self, additional=()):
        """Get recipients for BCC:

        May be empty."""

        return self._get_addresses('bccs', additional)

    def validate_addresses(self, sender=None, recipients=(), ccs=(), bccs=()):
        """Validates that there is one sender and at least one recipient"""
        sender = self.get_sender(sender)
        recipients = self.get_recipients(recipients, required=False)
        ccs = self.get_ccs(ccs)
        bccs = self.get_bccs(bccs)
        if not sender:
            raise MailrobotNoSenderError("Invalid Email: Lacks sender")
        if not (recipients | ccs | bccs):
            raise MailrobotNoRecipientsError("Invalid Email: Has no recipients")
        return True

    def make_message(self, sender=None, recipients=(), ccs=(), bccs=(), reply_to=None, headers=None, context=None):
        """Generate a django.core.mail.EmailMessage

        **sender** and **reply_to** may be overridden.
        **recipients**, **ccs** and **bccs** may be supplemented.

        Verfies that there is a sender and at least one recipient.
        """

        if not headers:
            headers = {}
        sender = self.get_sender(sender)
        reply_to = self.get_reply_to(reply_to)
        recipients = self.get_recipients(recipients)
        ccs = self.get_ccs(ccs)
        bccs = self.get_bccs(bccs)

        sendable = self.validate_addresses(sender, recipients, ccs, bccs)

        if reply_to:
            headers['Reply-To'] = reply_to

        subject = self.make_subject(context)
        content = self.make_content(context)
        message = EmailMessage(
            subject=subject,
            body=content,
            from_email=sender,
            to=recipients,
            cc=ccs,
            bcc=bccs,
            headers=headers
        )
        return message

    def send(self, **kwargs):
        "Use django's email backend system to send the Mail"

        message = self.make_message(**kwargs)
        message.send()



DEBUG = False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

SECRET_KEY = 'na2Tei0FoChe3ooloh5Yaec0ji7Aipho'

INSTALLED_APPS=(
    'mailrobot',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': False,
        'OPTIONS': {
            'debug': DEBUG,
        },
    },
]

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-09 12:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.EmailField(max_length=254, unique=True)),
                ('comment', models.CharField(blank=True, max_length=66, null=True)),
            ],
            options={
                'verbose_name_plural': 'addresses',
            },
        ),
        migrations.CreateModel(
            name='Mail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(max_length=40, unique=True)),
                ('bccs', models.ManyToManyField(blank=True, related_name='bcc', to='mailrobot.Address')),
                ('ccs', models.ManyToManyField(blank=True, related_name='cc', to='mailrobot.Address')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MailBody',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(max_length=40, unique=True)),
                ('subject', models.CharField(max_length=66)),
                ('body', models.TextField()),
            ],
            options={
                'verbose_name_plural': 'mailbodies',
            },
        ),
        migrations.CreateModel(
            name='Signature',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(max_length=40, unique=True)),
                ('sig', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='mail',
            name='content',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mail', to='mailrobot.MailBody'),
        ),
        migrations.AddField(
            model_name='mail',
            name='recipients',
            field=models.ManyToManyField(blank=True, related_name='recipients', to='mailrobot.Address'),
        ),
        migrations.AddField(
            model_name='mail',
            name='reply_to',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reply_to', to='mailrobot.Address'),
        ),
        migrations.AddField(
            model_name='mail',
            name='sender',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='sender', to='mailrobot.Address'),
        ),
        migrations.AddField(
            model_name='mail',
            name='signature',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='mail', to='mailrobot.Signature'),
        ),
    ]
